﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "LMItemDrop.h"

#include "Kismet/KismetMathLibrary.h"


ALMItemDrop::ALMItemDrop()
{
	PrimaryActorTick.bCanEverTick = true;
	Root = CreateDefaultSubobject<USceneComponent>("Root");
	RootComponent = Root;
	
	StaticMeshComponent = CreateDefaultSubobject<UStaticMeshComponent>("MeshComponent");
	StaticMeshComponent->SetupAttachment(RootComponent);
}

void ALMItemDrop::BeginPlay()
{
	Super::BeginPlay();
	SpawnLocation = GetActorLocation();
	GetWorldTimerManager().SetTimer(DespawnHandle, this, &ALMItemDrop::Despawn, TimeBeforeDespawn);
}

void ALMItemDrop::SetAttractor(AActor* Target)
{
	Attractor = Target;
	GetWorldTimerManager().SetTimer(MoveHandle, this, &ALMItemDrop::DoMovingTowardsTarget, MoveRate, true);
	GetWorldTimerManager().ClearTimer(DespawnHandle);
}

void ALMItemDrop::DoMovingTowardsTarget()
{
	if (MoveTime >= MoveDuration)
	{
		GetWorldTimerManager().ClearTimer(MoveHandle);
		SetActorLocation(Attractor->GetActorLocation());
		return;
	}
	
	const FVector Position = Attractor->GetActorLocation();

	const float Duration = MoveTime / MoveDuration;
	
	// FString text = FString::Printf(TEXT("Time : %f / %f = %f"), MoveTime, MoveDuration, Duration);
	// GEngine->AddOnScreenDebugMessage(-15, 1.f, FColor::Red, text);
	const float CurvePosition = PositionAtTime->GetFloatValue(Duration); 
	const FVector NewPosition = UKismetMathLibrary::VLerp(SpawnLocation, Position, CurvePosition);

	MoveTime += MoveRate;
	SetActorLocation(NewPosition);
}

void ALMItemDrop::ConsumePack(ALMPlayer* Player)
{
	if (MoveHandle.IsValid())
	{
		GetWorldTimerManager().ClearTimer(MoveHandle);
	}
}

void ALMItemDrop::Despawn()
{
	GetWorldTimerManager().ClearTimer(DespawnHandle);
	Destroy();
}

